<?php
// $Id: ora8_query.plib 769 2006-05-29 08:53:41Z vdlaarj $
// $URL: https://infix.uvt.nl/its-external/trunk/sources/uvtlib/ora8_query.plib $
/* Class oracle_query for Oracle 8
 * (C) 1998,1999,2000 CentER Applied Research, The Netherlands
 * (C) 2005 Tilburg University, The Netherlands
 *
 * This class serves as an application-independent interface to
 * Oracle functions in PHP. This class should not contain any GUI
 * element, please create a new class that either uses this class
 * or inherits it for GUI functions.
 *
 * Don't forget:
 * - OCIError() returns an array with keys 'code' and 'message'
 * - PHP-functions *usually* return 1 on success, our own functions
 *	 return 0 on success (C convention)!!!
 *
 */


  class oracle_query
  {
    var $query,           // contains the original query
        $conn,            // contains the oracle connection handle
        $stmt,            // contains the oracle statement, comparable to the cursor in Oracle 7
        $row,             // contains the active row
        $rows,            // contains the total number of rows
        $cols,            // contains the total number of columns
        $values,          // contains the return values of the query
        $lob,             // contains a LOB-descriptor (if any)
        $errormsg,        // contains the error message of the query (if any)
        $errorcode,       // contains the error code of the query (if any)
        $debug;           // more verbose error handling if set to 1


    /* Class constructor.
       On initialization of this class, the environment variables get set.

       <in>  user     = username
             password = password
             OSID     = database
       <out> Nothing or an error message if no connection established.
    */
    function oracle_query($user='', $password='', $OSID='')
    {
      if($OSID == '')
      {
        // If no database specfied, use the standard 'SU5DBI'
        $OSID = 'SU5DBI';
      }

      if($user != '')
      {
        //call connect
        $conn = $this->connect($user, $password, $OSID);

        if($conn)
        {
          return $this->error();
        }
      }
      //initialisation
      $this->row = -1;

    if (defined('DEBUG') && DEBUG == 1) {
      $this->debug = 1;
    } else {
      $this->debug = 0;
    }
  } // oracle_query


  /*
    Error handling, optional argument is a statement or a connection
    which defaults to '', this means the last error is presumed.

    <in>  stmt_conn = OPTIONAL statement or a connection
    <out> error message if DEBUG mode is activated
          and always the errorcode.
  */
  function error($stmt_conn = '') {
    $error = @ociError($stmt_conn);

    if (!$error) {
      // Hmm, there doesn't seem to be an error at all!
      return 0;
    } else {
      $this->errorcode = $error['code'];
      $this->errormsg  = $error['message'];

      // output if in debug-mode
      if ($this->debug) {
        echo '<div id="dberror">';
        printf('<p style="font-weight: bold; color: red;">Oracle error %s</p>',
               $this->errorcode);
        printf('<p>Query:<br />%s</p>',
               $this->query);
        printf('<p>Error:<br />%s</p>',
               $this->errormsg);
        echo '</div>';
      }

      // Return the errorcode in any case
      return $this->errorcode;
    }
  } // error


  /*
    Error message, optional argument is a statement or a connection
    Defaults to '', this means the last error is presumed.

    <in>  stmt_conn = statement or a connection
    <out> A string containing the error message,
          or the empty string if no error.
  */
  function errorMsg($stmt_conn="") {
    $error = @ociError($stmt_conn);

    if (!$error) {
      return '';
    }

    $this->errorcode = $error['code'];
    $this->errormsg  = $error['message'];

    return $this->errormsg;
  }  // errorMsg


  /*
    Connect to the specified oracle database with an username
    and password.
    <in>  user     = the username
          password = the password
          OSID     = the name of the database scheme
    <out> 0 on success, or an error message
  */
  function connect($user, $password, $OSID) {
    /*
      NOTE: ociNlogon is used instead of ocilogon, because
      otherwise the connection would be shared between different
      logons. This causes commits/rollbacks to apply to all
      open logons in one page. We don't want that.
    */

    $this->conn = ociNLogon($user, $password, $OSID);
    if (!$this->conn) {
      return $this->error($this->conn);
    }

    // Return 0 (==success)
    return 0;
  } // connect

  /*
    Disconnect from the oracle database.
    <in>  -
    <out> 0 on success, or an error message.
  */
  function disconnect() {
    // Free statement
    if (isset($this->stmt)) {
      @ocifreestatement($this->stmt);
    }

    // Log off from Oracle
    if (isset($this->conn)) {
      $logoff = @ociLogOff($this->conn);
    }
    if (!$logoff) {
      return $this->error($this->conn);
    }

    // Return 0 (==success)
    return 0;
  } // disconnect


  /*
    Bind an Oracle placeholder (':name') to a PHP variable.
    Must be called just before a select or execute. May be called
    more than once, but Oracle does not accept more than one buffer
    bigger than 4000 bytes.

    <in>  oraPH     = Oracle placeholder, such as ':myname'.
          phpVar    = PHP variable, including $ sign.
          maxLength = Optionally, maximum length of the string to transfer.
                      Oracle complains about 'String not ended with a zero'
                      when limit is reached. Yeah right.
                      By default, this parameter is the current length of
                      the phpVar.
    <out> -

  */
  function bind($oraPH, &$phpVar, $maxLength = 0) {
    if ($maxLength == 0) {
      $maxLength = strlen($phpVar)+1;
    }

    if (isset($this->bindings)) {
      $new = count($this->bindings);
    } else {
      $new = 0;
    }

    $this->bindings[$new]['ora'] = $oraPH;
    $this->bindings[$new]['php'] = $phpVar;
    $this->bindings[$new]['max'] = $maxLength;
  } // bind


  /*
    Execute a query on the database. User must connect first!

    <in>  query = the query to execute
    <out> 0 on success, or an error message.
  */
  function execute($query) {
    // First, destroy some variables from an older query, and
    // free the used statement (Oracle has a limit here).
    if (isset($this->stmt)) {
      @ocifreestatement($this->stmt);
    }
    unset($this->stmt);
    unset($this->query);
    unset($this->errorcode);
    unset($this->errormsg);

    // Set variable
    $this->query = $query;

    // Parse the query
    $this->stmt = @ociParse($this->conn, $query);
    if (!$this->stmt) {
      return $this->error($this->conn);
    }

    // Bind Oracle placeholders to PHP variables if requested.
    if (isset($this->bindings)) {
      for ($i=0; $i<sizeof($this->bindings); $i++) {
        @ociBindByName($this->stmt,
		       $this->bindings[$i]['ora'],
		       $this->bindings[$i]['php'],
  		       $this->bindings[$i]['max']);
      }
    }

    // Execute the query
    $result = @ociExecute($this->stmt);
    if (!$result) {
      return $this->error($this->stmt);
    }

    unset($this->bindings);

    // Return 0 (== success)
    return 0;
  } // execute


  /*
    Execute a select query on the database. User must connect first!

    <in>  query = the query
          mode  = OPTIONAL specifies the presentation of the output data.
                  Defaults to '', can be 'assoc'.
    <out> 0 on success, or an error message.
  */
  function select($query, $mode = '') {
    // First, destroy some variables which may effect a next query
    $this->cols = 0;
    $this->rows = 0;
    unset($this->row);
    unset($this->values);
    unset($this->errorcode);
    unset($this->errormsg);

    // Execute the query
    $result = $this->execute($query);
    if ($result) {
      return $result;
    }

    // Determine nr of cols
    $this->cols = @ociNumCols($this->stmt);
    if (!$this->cols) {
      return $this->error($this->stmt);
    }

    if ($mode == '') {
      // the default fetch mode
      // a numeric array with NULL values
      $mode = OCI_NUM+OCI_RETURN_NULLS;
    } elseif ($mode == 'assoc') {
      // fetch an associative array with NULL values
      $mode = OCI_ASSOC+OCI_RETURN_NULLS;
    }

    // Fetch all results in $this->values
    while(@ocifetchinto($this->stmt, $row, $mode)) {
      $this->values[] = $row;
      $this->rows++;
      unset($row);
    }

    $this->row = -1;
    if (!isset($this->values)) {
      // Nothing found, create an empty array
      $this->values = array();
    }

    // Return 0 (== success)
    return 0;
  } // select


  /*
    Returns the results of a query that has been executed before.
    It returns a two-dimensional array where the first index is the row
    identifier and the second index the column identifier.
  */
  function getResult() {
    return $this->values;
  } // getResult


  /*
    Returns the results of a query that has been executed before.
    It returns an array where the index is the number of the column in
    the specified row.

    If the row does not exist, the function returns false.
  */
  function getRow($row) {
    // Set internal row pointer
    $this->row = $row;

    if (empty($this->values[$row])) {
      return false;
    } else {
      return $this->values[$row];
    }
  } // getRow


  /*
    Returns the results of a query that has been executed before.
    It returns an array where the index is the number of the column in
    the specified row. The row is the row directly after the row that
    was accessed in a previous call to getRow or GetNextRow. The

    On failure, false is returned.
  */
  function getNextRow() {
    // Increase internal row pointer after current value is set in $row
    $row = ++$this->row;

    if (empty($this->values[$row])) {
      $this->row = -1;
      return false;
    }

    return $this->values[$this->row];
  } // getNextRow


  /*
    Returns the result of a query that has been executed before.
    It returns the value of the field on the specified row and column of
    the resultset.

    If the value does not exist, return false.
   */
   function getValue($row, $col) {
     if (empty($this->values[$row][$col])) {
       return false;
     } else {
       return $this->values[$row][$col];
     }
  } // getValue


  /*
    Returns the number of rows in the query that has been executed before.
  */
  function getNumberOfRows() {
    return $this->rows;
  }


  /*
    Return the number of columns in the query. The query must have been
    executed before, or you will get an Oracle error message.
  */
  function getNumberOfColumns() {
    return $this->cols;
  }


  /*
    Returns a string containing the errorcode and errormsg if any error
    occured or 'false' if no error occured.
  */
  function getError() {
    $return = '';

    if (isset($this->errorcode) and ($this->errorcode != '')) {
      $return .= $this->errorcode . ': ';
    }
    if (isset($this->errormsg) and ($this->errormsg != '')) {
      $return .= $this->errormsg;
    }

    if ($return != '') {
      return $return;
    } else {
      return false;
    }
  } // getError


  /*
    Data sanitizing functions, tuned to Oracle.
    <in>  string = the string to escape
    <out> the escaped string, ready for use within a query.
  */
  function escape_string($string) {
    // SQL-safe escapes of sensitive characters.
    // Single quote --> double single quote.
    $string = ereg_replace("'","''",$string);

    // I am not sure about the ':name' bind convention? ':' seems to work.
    // JH, 2006-01-23
    return $string;
  } // escape_string

} // class oracle_query

?>

